#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>

#define PROGNAME	"affinity"

int main(int argc, char **argv)
{
	DWORD dwSystemMask, dwCurrentMask, dwNewMask;
	HANDLE hProcess;
	BOOL fRandom = 0, fCPU = 0, fRun = 0;
	char *szCommand;
	long n;

	srand(GetTickCount());

	hProcess = GetCurrentProcess();

	GetProcessAffinityMask(hProcess, &dwCurrentMask, &dwSystemMask);
	dwNewMask = dwCurrentMask;

	for (; argc; argc--, argv++) {
		if (!strcmp(*argv, "--random")) {
			if (fCPU) {
				fprintf(stderr, PROGNAME": options --cpu and --random are incompatible\n");
				return 1;
			}

			fRandom = 1;

			do {
				n = rand() %32;
				dwNewMask = 1 << n;
			}
			while (!(dwSystemMask & dwNewMask));
		}

		if (!strcmp(*argv, "--cpu")) {
			if (fRandom) {
				fprintf(stderr, PROGNAME": options --cpu and --random are incompatible\n");
				return 1;
			} else if (!*++argv) {
				fprintf(stderr, PROGNAME": must specify a CPU with --cpu\n");
			}
			argc--;

			fCPU = 1;

			n = strtol(*argv, NULL, 10);

			if (errno == ERANGE) {
				fprintf(stderr, PROGNAME": must specify a CPU with --cpu\n");
				return 1;
			}

			dwNewMask = 1 << n;
			if (!(dwSystemMask & dwNewMask)) {
				fprintf(stderr, PROGNAME": CPU %d is not present in system\n", n);
				return 1;
			}
		}

		if (!strcmp(*argv, "--run")) {
			if (!*++argv) {
				fprintf(stderr, PROGNAME": must specify a command with --run\n");
				return 1;
			}
			argc--;

			szCommand = (char*)malloc(strlen(*argv) + 1);
			if (!szCommand) {
				perror(PROGNAME": malloc");
				return 1;
			}

			fRun = 1;

			strcpy(szCommand, *argv);
		} else if (!strcmp(*argv, "--help")) {
			printf(
"\n"
"affinity: runs a process on one CPU only\n"
"\n"
"    affinity [--random] [--cpu n] [--run command]\n"
"\n"
"    --random           selects a random CPU\n"
"    --cpu n            runs process on CPU n\n"
"    --run command      runs command, using cmd.exe if not specified\n"
"\n"
"(c) 1999 Chris Lightfoot <chris@ex-parrot.com> Free for redistribution.\n"
);
			return 0;
		}
	}

	SetProcessAffinityMask(hProcess, dwNewMask);

	if (fRun) WinExec(szCommand, SW_SHOWNORMAL);
	else WinExec("cmd.exe", SW_SHOWNORMAL);

	return 0;
}